/*
 * Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 * All rights reserved.
 *
 * http://www.real.com/devzone
 *
 * This program contains proprietary information of RealNetworks, Inc.,
 * and is licensed subject to restrictions on use and distribution.
 *
 * This file contains the methods for the client context object.
 */

#include "pntypes.h"
#include "pnwintyp.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmawin.h"
#include "rmaclsnk.h"
#include "rmaerror.h"
#include "os.h"
#include "auth.h"
#include "context.h"
#include "error.h"
#include "events.h"
#include "sitesup.h"

static tr_event_handler *event_handler = NULL;
tr_error_handler *error_handler = NULL;
static tr_auth_manager *auth_manager = NULL;
static tr_site_supplier *site_supplier = NULL;

tr_context::tr_context (void)
{
  error_handler = new tr_error_handler ();
  event_handler = new tr_event_handler ();
  auth_manager = new tr_auth_manager ();
  site_supplier = new tr_site_supplier ();
}

tr_context::~tr_context (void)
{
}

UINT32
tr_context::AddRef (void)
{
  return 1;
}

UINT32
tr_context::Release (void)
{
  return 1;
}

PN_RESULT
tr_context::QueryInterface (const IID &id, void **obj)
{
  if (IsEqualIID (id, IID_IRMAClientAdviseSink))
    {
      *obj = event_handler;
      return PNR_OK;
    }
  else if (IsEqualIID (id, IID_IRMAErrorSink))
    {
      *obj = error_handler;
      return PNR_OK;
    }
  else if (IsEqualIID (id, IID_IRMAAuthenticationManager))
    {
      *obj = auth_manager;
      return PNR_OK;
    }
  else if (IsEqualIID (id, IID_IRMASiteSupplier))
    {
      *obj = site_supplier;
      return PNR_OK;
    }
  else if (IsEqualIID (id, IID_IUnknown))
    {
      *obj = this;
      return PNR_OK;
    }

  *obj = NULL;
  return PNR_NOINTERFACE;
}
