/*
 * Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 * All rights reserved.
 *
 * http://www.real.com/devzone
 *
 * This program contains proprietary information of RealNetworks, Inc.,
 * and is licensed subject to restrictions on use and distribution.
 *
 * $Id: events.cc,v 1.2 2002/08/04 15:25:33 paul Exp $
 *
 * This file contains the methods for the event handler.
 */

#include <math.h>
#include "pntypes.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "rmacore.h"
#include "os.h"
#include "rmaclsnk.h"
#include "events.h"
#include "tac.h"
#include "main.h"

tr_event_handler::tr_event_handler (void)
{
  player->AddAdviseSink (this);
}

tr_event_handler::~tr_event_handler (void)
{
}

PN_RESULT
tr_event_handler::OnPosLength (UINT32 position, UINT32 new_length)
{
  if (player->IsLive ())
    new_length = 0;

  if (length != (INT32) new_length)
    {
      length = new_length;

      if (flags & SHOW_LENGTH)
	print_length ();
    }

  return PNR_OK;
}

PN_RESULT
tr_event_handler::OnPresentationOpened (void)
{
  return PNR_OK;
}

PN_RESULT
tr_event_handler::OnPresentationClosed (void)
{
  return PNR_OK;
}

PN_RESULT
tr_event_handler::OnStatisticsChanged (void)
{
  INT32 new_bandwidth;

  registry->GetIntByName ("Statistics.Player0.ClipBandwidth", new_bandwidth);

  if ((UINT32) new_bandwidth != bandwidth)
    {
      bandwidth = (UINT32) new_bandwidth;

      if (flags & SHOW_BANDWIDTH)
	print_bandwidth ();
    }

  return PNR_OK;
}

PN_RESULT
tr_event_handler::OnPreSeek (UINT32 old_time, UINT32 new_time)
{
  flags |= SEEKING;
  return PNR_OK;
}

PN_RESULT
tr_event_handler::OnPostSeek (UINT32 old_time, UINT32 new_time)
{
  flags &= ~SEEKING;
  return PNR_OK;
}

PN_RESULT
tr_event_handler::OnStop (void)
{
  /* Turn on the STOPPED flag and clear all of the other state-related
     flags. */
  flags = (flags | STOPPED) & ~(CONTACTING | BUFFERING | PLAYING | PAUSED
				| SEEKING);
  return PNR_OK;
}

PN_RESULT
tr_event_handler::OnPause (UINT32 time)
{
  flags |= PAUSED;
  return PNR_OK;
}

PN_RESULT
tr_event_handler::OnBegin (UINT32 time)
{
  update_tac ();

  if ((flags & SHOW_TAC) && (~flags & PAUSED))
    print_tac ();

  if (flags & PAUSED)
    flags &= ~PAUSED;
  else
    flags = (flags & ~CONTACTING) | PLAYING;

  return PNR_OK;
}

PN_RESULT
tr_event_handler::OnBuffering (UINT32 buf_flags, UINT16 percent)
{
  /* The CONTACTING flag should not be set when the BUFFERING flag is set. */
  flags &= ~CONTACTING;
  flags = (percent < 100) ? (flags | BUFFERING) : (flags & ~BUFFERING);
  buffer_percent = percent;
  static UINT16 last_printed_percent = 0;

  if ((abs (percent - last_printed_percent) >= 10) && (percent < 100) &&
      ((~flags & QUIET) && (~flags & SHOW_STATUS_LINE)))
    {
      print_buffering_info ();
      last_printed_percent = percent;
    }

  return PNR_OK;
}

PN_RESULT
tr_event_handler::OnContacting (const char *host_name)
{
  if (~flags & CONTACTING)
    {
      flags |= CONTACTING;

      if ((~flags & QUIET) && (~flags & SHOW_STATUS_LINE))
	print_state ();
    }

  return PNR_OK;
}

UINT32
tr_event_handler::AddRef (void)
{
  return 1;
}

UINT32
tr_event_handler::Release (void)
{
  return 1;
}

PN_RESULT
tr_event_handler::QueryInterface (const IID &id, void **obj)
{
  if (IsEqualIID (id, IID_IRMAClientAdviseSink) ||
      IsEqualIID (id, IID_IUnknown))
    {
      *obj = this;
      return PNR_OK;
    }

  *obj = NULL;
  return PNR_NOINTERFACE;
}
