/*
 * Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 * All rights reserved.
 *
 * http://www.real.com/devzone
 *
 * This program contains proprietary information of RealNetworks, Inc.,
 * and is licensed subject to restrictions on use and distribution.
 *
 * $Id: main.cc,v 1.2 2002/08/04 15:25:33 paul Exp $
 */

#define INITGUID

#include <stdlib.h>
#include <signal.h>
#include <slang.h>
#include <dlfcn.h>
#include <time.h>
#include "pntypes.h"
#include "pncom.h"
#include "rmapckts.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "rmacore.h"
#include "rmaclsnk.h"
#include "rmaerror.h"
#include "rmaauth.h"
#include "rmawin.h"
#include "rmagroup.h"
#include "os.h"
#include "context.h"
#include "error.h"
#include "main.h"
#include "tac.h"

#define REALPLAYER_HOME "/usr/lib/RealPlayer8"

typedef PN_RESULT (PNEXPORT_PTR FPRMSETDLLACCESSPATH) (const char *);

unsigned short flags1 = INTERACTIVE;

INT32 length1 = -1;
UINT32 error1 = PNR_OK, bandwidth1 = 0;
UINT16 buffer_percent1 = 0;
IRMAPlayer *player1 = NULL;
IRMAPNRegistry *registry1 = NULL;

/* Print an amount of time specified in milliseconds.  If the compact_time
   flag is set, print the time in "hh:mm:ss" format.  Otherwise, print it
   out in "X hours X minutes and X seconds" format. */
static void
print_time (UINT32 time, unsigned char compact_time)
{
  int hours = time / 3600000;
  int minutes = (time % 3600000) / 60000;
  int seconds = (time % 60000) / 1000;

  if (compact_time)
    printf ("%02d:%02d:%02d", hours, minutes, seconds);
  else
    {
      if (hours)
	{
	  printf ("%d %s", hours, hours == 1 ? "hour" : "hours");

	  if (minutes || seconds)
	    {
	      putchar (' ');

	      if (minutes && !seconds)
		fputs ("and ", stdout);
	    }
	}

      if (minutes)
	printf ("%d %s", minutes, minutes == 1 ? "minute" : "minutes");

      if (seconds)
	{
	  if (hours || minutes)
	    fputs (" and ", stdout);

	  printf ("%d %s", seconds, seconds == 1 ? "second" : "seconds");
	}
    }
}

void
init_tty (void)
{
  SLang_TT_Read_FD = 0;

  if (isatty (0) && (SLtt_initialize (NULL) == 0) &&
      (SLang_init_tty (-1, 1, 1) != -1) &&
      (SLkp_init () != -1))
    flags1 |= TTY_INITIALIZED;
}

/* If the terminal is initialized and the status line is being continuously
   shown, clear the current line on the terminal.  Otherwise, do nothing. */
static void
clear_status_line (void)
{
  if ((flags1 & TTY_INITIALIZED) && (flags1 & SHOW_STATUS_LINE))
    {
      SLtt_erase_line ();
      SLtt_flush_output ();
    }
}

void
reset_tty (void)
{
  if (flags1 & TTY_INITIALIZED)
    {
      clear_status_line ();
      SLang_reset_tty ();
      flags1 &= ~TTY_INITIALIZED;
    }
}

void
print_length (void)
{
  unsigned char compact = flags1 & COMPACT_OUTPUT;
  clear_status_line ();

  if (player1->IsLive ())
    puts ("Live program");
  else if (length1)
    {
      if (compact)
	fputs ("Length: ", stdout);
      else
	fputs("Length is ", stdout);

      print_time (length1, compact);
      putchar ('\n');
    }
}

void
print_bandwidth (void)
{
  clear_status_line ();
  printf ("Bit rate is %gK\n", bandwidth1 / 1000.0);
}

void
print_tac (void)
{
  clear_status_line();

  for (int i = 0; i < NUM_TAC_PROPS; i++)
    {
      if (tac_props[i] && *tac_props[i])
	printf ("%s: %s\n", tac_names[i], tac_props[i]);
    }
}

/* Return a word indicating the state of playback. */
static const char *
get_state_string (void)
{
  if (flags1 & CONTACTING)
    return "Contacting";
  else if (flags1 & PAUSED)
    return "Paused";
  else if (flags1 & BUFFERING)
    return "Buffering";
  else if (flags1 & SEEKING)
    return "Seeking";
  else if (flags1 & PLAYING)
    return "Playing";
  else if (flags1 & STOPPED)
    return "Stopped";
  else
    return "Starting";
}

void
print_buffering_info (void)
{
  printf ("Buffering (%d%%)\n", buffer_percent1);
}

void
print_state (void)
{
  clear_status_line ();

  if ((flags1 & BUFFERING) && (~flags1 & PAUSED))
    print_buffering_info ();
  else
    puts (get_state_string ());
}

