/*
 * $Id: main.h,v 1.2 2002/08/04 15:25:33 paul Exp $
 *
 * This file contains the definitions and function prototypes for the main
 * program.
 */

#ifndef _MAIN_H_
#define _MAIN_H_

#include "rmacore.h"
#include "rmamon.h"

/* The following are flags that are used throughout the player. */
#define SHOW_BANDWIDTH 1
#define SHOW_TAC 2
#define SHOW_LENGTH 4
#define QUIET 8
#define COMPACT_OUTPUT 16
#define INTERACTIVE 32
#define CONTACTING 64
#define BUFFERING 128
#define PLAYING 256
#define PAUSED 512
#define SEEKING 1024
#define STOPPED 2048
#define TTY_INITIALIZED 4096
#define QUIT 8192
#define UNKNOWN_OPTION 16384
#define SHOW_STATUS_LINE 32768

/* This variable indicates the settings of the flags defined above. */
extern unsigned short flags;

/* This variable indicates the length of the clip.  If it is negative, the
   length has not yet been determined.  If it is zero, it is either a live
   program or a zero-length clip. */
extern INT32 length;

/* This variable contains the RealSystem error code of the last error that
   occurred. */
extern UINT32 error;

/* This variable indicates the current bandwidth of the current clip.  If it
   is 0, the bandwidth has not yet been determined. */
extern UINT32 bandwidth;

/* This variable indicates the percentage of the buffer that has been filled.
   It is only valid when the player is buffering. */
extern UINT16 buffer_percent;

/* This variable points to the player object from the client core. */
extern IRMAPlayer *player;

/* This variable points to the registry object from the client core. */
extern IRMAPNRegistry *registry;

extern "C"
{
  /* Print the current clip's length in a format determined by the
     COMPACT_OUTPUT flag. */
  extern void print_length (void);

  /* Print the current bandwidth of the current clip. */
  extern void print_bandwidth (void);

  /* Print the title, author, copyright, and possibly other info for the
     current clip. */
  extern void print_tac (void);

  /* Print the word "Buffering" followed by the percent of the buffer that
     has been filled. */
  extern void print_buffering_info (void);

  /* Print the state of playback. */
  extern void print_state (void);

  /* Determine whether the terminal is VT100-compatible.  If so, initialize
     it to read single keystrokes and set the TTY_INITIALIZED flag. */
  extern void init_tty (void);

  /* If the terminal has been initialized, reset it and clear the
     TTY_INITIALIZED flag. */
  extern void reset_tty (void);
}

#endif
