/*
 * Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 * All rights reserved.
 *
 * http://www.real.com/devzone
 *
 * This program contains proprietary information of RealNetworks, Inc.,
 * and is licensed subject to restrictions on use and distribution.
 *
 * $Id: map.c,v 1.2 2002/08/04 15:25:33 paul Exp $
 *
 * This file contains the functions that operate on a map structure.
 */

#include "os.h"
#include "map.h"

#define SIZE_INCREMENT 10

void
init_map (map_t *map)
{
  map->keys = NULL;
  map->values = NULL;
  map->size = 0;
  map->alloc_size = 0;
}

int
map_get (const map_t *map, void *key, void **value)
{
  int i;

  for (i = 0; i < map->size; i++)
    if (map->keys[i] == key)
      {
	*value = map->values[i];
	return 1;
      }

  return 0;
}

void
map_remove (map_t *map, void *key)
{
  int i;

  for (i = 0; i < map->size; i++)
    if (map->keys[i] == key)
      {
	if (i < map->size - 1)
	  {
	    memmove (&map->keys[i], &map->keys[i + 1],
		     sizeof (void *) * (map->size - i + 1));
	    memmove (&map->values[i], &map->values[i + 1],
		     sizeof (void *) * (map->size - i + 1));
	  }

	map->size--;
	return;
      }
}

void
map_put (map_t *map, void *key, void *value)
{
  int i;

  for (i = 0; i < map->size; i++)
    if (map->keys[i] == key)
      {
	map->values[i] = value;
	return;
      }

  if (map->size == map->alloc_size)
    {
      void **new_keys, **new_values;

      map->alloc_size += SIZE_INCREMENT;
      new_keys = (void **) malloc (sizeof (void *) * map->alloc_size);
      new_values = (void **) malloc (sizeof (void *) * map->alloc_size);

      if (map->keys != NULL)
	{
	  memcpy (new_keys, map->keys, sizeof (void *) * map->size);
	  memcpy (new_values, map->values, sizeof (void *) * map->size);
	  free (map->keys);
	  free (map->values);
	}

      map->keys = new_keys;
      map->values = new_values;
    }

  map->keys[map->size] = key;
  map->values[map->size] = value;
  map->size++;
}
