/*
 * Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 * All rights reserved.
 *
 * http://www.real.com/devzone
 *
 * This program contains proprietary information of RealNetworks, Inc.,
 * and is licensed subject to restrictions on use and distribution.
 *
 * $Id: site.cc,v 1.2 2002/08/04 15:25:33 paul Exp $
 *
 * This file contains the methods for the site object.
 */

#include "pncom.h"
#include "pntypes.h"
#include "pnwintyp.h"
#include "rmawin.h"
#include "rmaevent.h"
#include "rmacomm.h"
#include "rmapckts.h"
#include "os.h"
#include "main.h"
#include "site.h"

tr_site::tr_site (void): user (NULL), values (NULL), watcher (NULL)
{
  IRMACommonClassFactory *ccf;
  player->QueryInterface (IID_IRMACommonClassFactory, (void **) &ccf);
  ccf->CreateInstance (CLSID_IRMAValues, (void **) &values);
  ccf->Release ();
  size.cx = 0;
  size.cy = 0;
  position.x = 0;
  position.y = 0;
}

tr_site::~tr_site (void)
{
  values->Release ();
}

PN_RESULT
tr_site::QueryInterface (const IID &id, void **obj)
{
  if (IsEqualIID (id, IID_IRMASite) ||
      IsEqualIID (id, IID_IUnknown))
    {
      *obj = this;
      return PNR_OK;
    }
  else if (IsEqualIID (id, IID_IRMAValues))
    {
      values->AddRef ();
      *obj = values;
      return PNR_OK;
    }

  *obj = NULL;
  return PNR_NOINTERFACE;
}

UINT32
tr_site::AddRef (void)
{
  return 1;
}

UINT32
tr_site::Release (void)
{
  return 1;
}

PN_RESULT
tr_site::AttachUser (IRMASiteUser *new_user)
{
  if (user)
    return PNR_UNEXPECTED;

  PN_RESULT result = new_user->AttachSite (this);
    
  if (result == PNR_OK)
    {
      user = new_user;
      user->AddRef ();
    }

  return result;
}

PN_RESULT
tr_site::DetachUser (void)
{
  if (!user)
    return PNR_UNEXPECTED;

  PN_RESULT result = user->DetachSite ();
    
  if (result == PNR_OK)
    {
      user->Release ();
      user = NULL;
    }

  return result;
}

PN_RESULT
tr_site::GetUser (IRMASiteUser *&current_user)
{
  if (!user)
    return PNR_UNEXPECTED;

  current_user = user;
  user->AddRef ();
  return PNR_OK;
}

PN_RESULT
tr_site::CreateChild (IRMASite *&child)
{
  child = new tr_site ();
  return PNR_OK;
}

PN_RESULT
tr_site::DestroyChild (IRMASite *child)
{
  delete child;
  return PNR_OK;
}

PN_RESULT
tr_site::AttachWatcher (IRMASiteWatcher *new_watcher)
{
  if (watcher)
    return PNR_UNEXPECTED;

  watcher = new_watcher;

  if (watcher)
    {
      watcher->AddRef ();
      watcher->AttachSite (this);
    }

  return PNR_OK;
}

PN_RESULT
tr_site::DetachWatcher (void)
{
  if (!watcher)
    return PNR_UNEXPECTED;

  watcher->DetachSite ();
  PN_RELEASE (watcher);
  return PNR_OK;
}

PN_RESULT
tr_site::SetSize (PNxSize new_size)
{
  PN_RESULT result = PNR_OK;

  if (watcher)
    result = watcher->ChangingSize (size, new_size);

  if (result == PNR_OK && new_size.cx != 0 && new_size.cy != 0)
    size = new_size;

  return result;
}

PN_RESULT
tr_site::SetPosition (PNxPoint new_position)
{
  PN_RESULT result = PNR_OK;

  if (watcher)
    result = watcher->ChangingPosition (position, new_position);
    
  if (result == PNR_OK)
    position = new_position;

  return result;
}

PN_RESULT
tr_site::GetSize (PNxSize &current_size)
{
  current_size = size;
  return PNR_OK;
}

PN_RESULT
tr_site::GetPosition (PNxPoint &current_position)
{
  current_position = position;
  return PNR_OK;
}

PN_RESULT
tr_site::DamageRect (PNxRect rect)
{
  return PNR_OK;
}

PN_RESULT
tr_site::DamageRegion (PNxRegion region)
{
  return PNR_OK;
}

PN_RESULT
tr_site::ForceRedraw (void)
{
  return PNR_OK;
}
