/*
 * Copyright (C) 1995,1996,1997 RealNetworks.
 * All rights reserved.
 *
 * This program contains proprietary information of RealNetworks, Inc.,
 * and is licensed subject to restrictions on use and distribution.
 *
 * $Id: sitesup.cc,v 1.2 2002/08/04 15:25:33 paul Exp $
 *
 * This file contains the methods for the site supplier.
 */

#include "pncom.h"
#include "pntypes.h"
#include "pnwintyp.h"
#include "rmawin.h"
#include "rmapckts.h"
#include "rmacomm.h"
#include "main.h"
#include "map.h"
#include "site.h"
#include "sitesup.h"

static IRMASiteManager *site_manager = NULL;
static map_t created_sites;

tr_site_supplier::tr_site_supplier (void)
{
  player->QueryInterface (IID_IRMASiteManager, (void **) &site_manager);
  init_map (&created_sites);
}

tr_site_supplier::~tr_site_supplier (void)
{
  PN_RELEASE (site_manager);
}

PN_RESULT
tr_site_supplier::SitesNeeded (UINT32 request_id, IRMAValues *props)
{
  if (!props)
    return PNR_INVALID_PARAMETER;

  PN_RESULT result = PNR_OK;
  IRMAValues *site_props = NULL;
  IRMABuffer *value = NULL;
  IRMASite *site = NULL;
  site = new tr_site ();
  site->QueryInterface (IID_IRMAValues, (void **) &site_props);
  result = props->GetPropertyCString ("playto", value);

  if (result == PNR_OK)
    {
      site_props->SetPropertyCString ("channel", value);
      PN_RELEASE (value);
    }
  else
    {
      result = props->GetPropertyCString ("name", value);

      if (result == PNR_OK)
	{
	  site_props->SetPropertyCString ("LayoutGroup", value);
	  PN_RELEASE (value);
	}
    }

  site_props->Release ();
  result = site_manager->AddSite (site);

  if (result != PNR_OK)
    {
      delete site;
      return result;
    }

  map_put (&created_sites, (void *) request_id, site);
  return PNR_OK;
}

PN_RESULT
tr_site_supplier::SitesNotNeeded (UINT32 request_id)
{
  IRMASite *site = NULL;

  if (!map_get (&created_sites, (void *) request_id, (void **) &site))
    return PNR_INVALID_PARAMETER;

  site_manager->RemoveSite (site);
  delete site;
  map_remove (&created_sites, (void *) request_id);
  return PNR_OK;
}

PN_RESULT
tr_site_supplier::BeginChangeLayout (void)
{
  return PNR_OK;
}

PN_RESULT
tr_site_supplier::DoneChangeLayout (void)
{
  return PNR_OK;
}

UINT32
tr_site_supplier::AddRef (void)
{
  return 1;
}

UINT32
tr_site_supplier::Release (void)
{
  return 1;
}

PN_RESULT
tr_site_supplier::QueryInterface (const IID &id, void **obj)
{
  if (IsEqualIID (id, IID_IRMASiteSupplier) ||
      IsEqualIID (id, IID_IUnknown))
    {
      *obj = this;
      return PNR_OK;
    }

  *obj = NULL;
  return PNR_NOINTERFACE;
}
