/*
 * Copyright (C) 1995,1996,1997 RealNetworks, Inc.
 * All rights reserved.
 *
 * http://www.real.com/devzone
 *
 * This program contains proprietary information of RealNetworks, Inc.,
 * and is licensed subject to restrictions on use and distribution.
 *
 * $Id: tac.cc,v 1.2 2002/08/04 15:25:33 paul Exp $
 *
 * This file contains functions and variables related to the title, author,
 * and copyright info for a clip.
 */

#include "pncom.h"
#include "pntypes.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "rmaengin.h"
#include "rmacore.h"
#include "rmapckts.h"
#include "os.h"
#include "main.h"
#include "tac.h"

const char *tac_names[] = {"Title", "Author", "Copyright", "Abstract",
			   "Keywords"};
static UINT32 tac_ids[NUM_TAC_PROPS] = {0};
char *tac_props[NUM_TAC_PROPS] = {NULL};

void
init_tac (void)
{
  IRMARegistryID *reg_id = NULL;
  UINT32 player_index;
  player->QueryInterface (IID_IRMARegistryID, (void **) &reg_id);
  reg_id->GetID (player_index);
  reg_id->Release ();
  IRMABuffer *buffer = NULL;
  registry->GetPropName(player_index, buffer);
  UCHAR *reg_name = buffer->GetBuffer ();
  char name[MAX_DISPLAY_NAME] = "";

  for (int n = 0; n < NUM_TAC_PROPS; n++)
    {
      sprintf (name, "%s.%s", reg_name, tac_names[n]);
      tac_ids[n] = registry->GetId (name);
    }

  buffer->Release ();
  update_tac ();
}

/* Change the value of a property, freeing the old value if necessary. */
static void
set_prop (char **prop, const char *value)
{
  if (*prop)
    free (*prop);

  *prop = strdup (value);
}

void
update_tac (void)
{
  IRMABuffer *value = NULL;

  for (int n = 0; n < NUM_TAC_PROPS; n++)
    {
      registry->GetStrById (tac_ids[n], value);

      if (value)
	{
	  set_prop (&tac_props[n], (const char *) value->GetBuffer ());
	  value->Release ();
	  value = NULL;
	}
    }
}
