/***********************************************************************
 *   Code is based on wmppp, wmload, wmtime, wmcp, and asbutton
 *   Author: Edward H. Flora <ehflora@ksu.edu>
 *   Ver 0 Rel 4    March 6, 1999
 *
 *   Contributors:
 *              Ben Cohen <buddog@aztec.asu.edu>
 *                  original author of wmcp (et al.)
 *              Thomas Nemeth <tnemeth@multimania.com>
 *                  contributor to wmcp
 *              Casey Harkins <charkins@cs.wisc.edu> 
 *                  Bug fix reading config file path - 3/6/99
 *                  Added button-presses, and other - denoted by *charkins*
 *              Michael Henderson <mghenderson@lanl.gov>
 *                  Application ideas, suggestions
 *              Ryan ?? <pancake@mindspring.com> 
 *                  Modified wmbutton to asbutton.
 *                  Note: asbutton is a seperate program, not associated 
 *                        with wmbutton (just as wmbutton is not associated
 *                        with wmcp)
 *              Jon Bruno
 *                  Web Page Development
 *    The contributors listed above are not necessarily involved with the 
 *    development of wmbutton.  I'm listing them here partially as thanks for 
 *    helping out, catching bugs in the code, etc.
 ***********************************************************************/
#include <Xlib.h>
#include <Xutil.h>
#include <xpm.h>
#include <extensions/shape.h>
#include <keysym.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <signal.h>
#include <dlfcn.h>
#include <time.h>
#include <slang.h>
#include "pntypes.h"
#include "pncom.h"
#include "rmapckts.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "rmacore.h"
#include "rmaclsnk.h"
#include "rmaerror.h"
#include "rmaauth.h"
#include "rmawin.h"
#include "rmagroup.h"
#include "os.h"
#include "context.h"
#include "error.h"
#include "main.h"
#include "tac.h"

#include "backdrop.xpm"		// background graphic
#include "wmreal.xpm"		// graphic of 9 buttons
#include "mask.xbm"

#define CONFIGFILEMAX 128
#define CONFFILENAME  "/.wmreal"
#define CONFIGGLOBAL  "/etc/wmreal.conf"	// added for Debianization
#define VER           0
#define REL           4

// #define MIDMOUSE
#define LMASK         0
#define MMASK         10
#define RMASK         20

#define NUMAPPS       9

#define REALPLAYER_HOME "/usr/lib/RealPlayer8"

unsigned short flags = INTERACTIVE;
static time_t run_time = -1;
static UINT32 seek_position;
static UINT32 quit_time;


INT32 length = -1;
UINT32 error = PNR_OK, bandwidth = 0;
UINT16 buffer_percent = 0;
IRMAPlayer *player = NULL;
IRMAPNRegistry *registry = NULL;

static void interrupted(int signal)
{
    flags |= QUIT;
}


typedef PN_RESULT(PNEXPORT_PTR FPRMSETDLLACCESSPATH) (const char *);

#define URL "rtsp://media1.abc.net.au/encoder/triplej.rm"

/***********************************************************************
 * 		Globals..    OK.. there's too many globals.
 *                           Feel free and fix it, if you'd like.
 ***********************************************************************/
Display *display;
Window rootwin, win, iconwin;
GC gc;
Pixel bg_pixel, fg_pixel;

char configfile[CONFIGFILEMAX];

typedef struct _XpmIcon {
    Pixmap pixmap;
    Pixmap mask;
    XpmAttributes attributes;
} XpmIcon;


Pixmap pixmask;

typedef struct _button_region {
    int x, y;
    int i, j;
} ButtonArea;

ButtonArea button_region[9];

XpmIcon templt, visible, buttons;

//int numb_of_apps = 9;
int border = 0;
int Verbose = 0;
int button_pressed = -1;	/* button to be drawn pressed *charkins */

char *app_name = "wmreal";

/*************** Function Prototypes ***********************************/
void RunAppN(int app);		// function to run app N as found in conf file
char *Parse(int app);		// parse data in config file
void redraw(void);
void getPixmaps(void);
int whichButton(int x, int y);	// determine which button has been pressed
int flush_expose(Window w);
void show_usage(void);		// show usage message to stderr
char *readln(FILE * fp);	// read line from file, return pointer to it


/* This function handles commands that move from one clip to another.  It
   returns 1 if the command succeeded or 0 if it didn't. */
static int clip_motion_command(int key)
{
    static IRMAGroupManager *group_manager = NULL;
    UINT16 group_count, current_group;

    if (group_manager == NULL)
	player->QueryInterface(IID_IRMAGroupManager,
			       (void **) &group_manager);

    group_count = group_manager->GetGroupCount();

    if (group_manager->GetCurrentGroup(current_group) != PNR_OK)
	return 0;

    switch (key) {
    case '[':
	return (current_group > 0)
	    && (group_manager->SetCurrentGroup(current_group - 1) ==
		PNR_OK);
    case ']':
	return (current_group < group_count - 1)
	    && (group_manager->SetCurrentGroup(current_group + 1) ==
		PNR_OK);
    case '{':
	return (current_group > 0)
	    && (group_manager->SetCurrentGroup(0) == PNR_OK);
    case '}':
	return (current_group < group_count - 1)
	    && (group_manager->SetCurrentGroup(group_count - 1) == PNR_OK);
    default:
	/* This should never happen. */
	return 0;
    }
}


/* Print an amount of time specified in milliseconds.  If the compact_time
   flag is set, print the time in "hh:mm:ss" format.  Otherwise, print it
   out in "X hours X minutes and X seconds" format. */
static void print_time(UINT32 time, unsigned char compact_time)
{
    int hours = time / 3600000;
    int minutes = (time % 3600000) / 60000;
    int seconds = (time % 60000) / 1000;

    if (compact_time)
	printf("%02d:%02d:%02d", hours, minutes, seconds);
    else {
	if (hours) {
	    printf("%d %s", hours, hours == 1 ? "hour" : "hours");

	    if (minutes || seconds) {
		putchar(' ');

		if (minutes && !seconds)
		    fputs("and ", stdout);
	    }
	}

	if (minutes)
	    printf("%d %s", minutes, minutes == 1 ? "minute" : "minutes");

	if (seconds) {
	    if (hours || minutes)
		fputs(" and ", stdout);

	    printf("%d %s", seconds, seconds == 1 ? "second" : "seconds");
	}
    }
}

/* Return a word indicating the state of playback. */
static const char *get_state_string(void)
{
    if (flags & CONTACTING)
	return "Contacting";
    else if (flags & PAUSED)
	return "Paused";
    else if (flags & BUFFERING)
	return "Buffering";
    else if (flags & SEEKING)
	return "Seeking";
    else if (flags & PLAYING)
	return "Playing";
    else if (flags & STOPPED)
	return "Stopped";
    else
	return "Starting";
}

/* Print a status line.  If the new_line flag is set, go to a new line when
   done.  Otherwise, clear the rest of the line and move the cursor back
   to the start of the line. */
static void print_status_line(unsigned char new_line)
{
    if ((~flags & CONTACTING) && (~flags & BUFFERING) && (~flags & PLAYING)
	&& (~flags & PAUSED) && (~flags & SEEKING) && (~flags & STOPPED))
	/* It's just getting started, so don't print a status line yet. */
	return;

    printf("%-7s", get_state_string());

    if (~flags & CONTACTING) {
	if ((flags & BUFFERING) && (~flags & PAUSED))
	    printf("      %3d%%", buffer_percent);
	else if (bandwidth == 0)
	    /* The bandwidth hasn't been determined yet. */
	    fputs("            ", stdout);
	else {
	    fputs("  Rate: ", stdout);

	    if (bandwidth < 1000)
		printf("%4d", bandwidth);
	    else {
		int unit_size;
		char suffix;
		double divided_bandwidth;

		if (bandwidth >= 1000 && bandwidth < 1000000) {
		    unit_size = 1000;
		    suffix = 'K';
		} else {
		    unit_size = 1000000;
		    suffix = 'M';
		}

		divided_bandwidth = bandwidth / (double) unit_size;

		if (divided_bandwidth < 10
		    && ((int) divided_bandwidth != divided_bandwidth))
		    printf("%3.1f", divided_bandwidth);
		else
		    printf("%3d", (int) divided_bandwidth);

		fputc(suffix, stdout);
	    }
	}

	fputs("  Time: ", stdout);
	UINT32 play_time;

	/* When the player stops, the client core sets its playback time to 0.
	   However, this is not what we want when the status line is being
	   displayed for the command line.  So if the player was stopped because
	   the QUIT flag was set, we use the quit_time variable.  If the player
	   reached the end of the presentation, we use the length variable. */
	if (flags & QUIT)
	    play_time = quit_time;
	else if (flags & STOPPED)
	    play_time = length;
	else
	    play_time = player->GetCurrentPlayTime();

	print_time(play_time, 1);

	if (player->IsLive())
	    fputs(" Live", stdout);
	else if (length > 0) {
	    printf(" %3d%% of ",
		   (int) (play_time / (double) length * 100));
	    print_time(length, 1);
	}
    }

    if (new_line)
	putchar('\n');
    else {
	SLtt_del_eol();
	putchar('\r');
    }
}

/* If the terminal is initialized and the status line is being continuously
   shown, clear the current line on the terminal.  Otherwise, do nothing. */
static void clear_status_line(void)
{
    if ((flags & TTY_INITIALIZED) && (flags & SHOW_STATUS_LINE)) {
	SLtt_erase_line();
	SLtt_flush_output();
    }
}

/* Seek to an absolute position in the current clip and set the seek_position
   variable to that position, which should be in milliseconds.  Return 1 if
   successful or 0 if not. */
static int seek_absolute(int position)
{
    if (player->Seek(position) == PNR_OK) {
	seek_position = position;
	return 1;
    } else
	return 0;
}


/* Move a certain number of seconds in the presentation.  If the number is
   positive, move forward; if negative, move backward.  Return 1 if successful
   or 0 if not. */
static int seek(int amount)
{
    INT32 time = (flags & SEEKING) ? seek_position
	: player->GetCurrentPlayTime();
    time += amount * 1000;

    if ((time >= 0) && (time <= length) && !player->IsLive())
	return seek_absolute(time);
    else
	return 0;
}

/* This function runs a command that the user has entered from the keyboard.
   It returns 1 if the command succeeded or 0 if it didn't. */
static int run_command(int key)
{
    int check_position = (flags & SEEKING) ? seek_position
	: player->GetCurrentPlayTime();

    switch (key) {
    case 'p':
	if (flags & PAUSED)
	    return (player->Begin() == PNR_OK);
	else
	    return (player->Pause() == PNR_OK);
    case 'e':
	flags |= QUIT;
	return 1;
    case 26:
	reset_tty();
	raise(SIGTSTP);
	init_tty();
	return 1;
    case SL_KEY_LEFT:
    case ',':
	return seek(-6);
    case SL_KEY_RIGHT:
    case '.':
	return seek(6);
    case SL_KEY_UP:
    case '<':
	return seek(-60);
    case SL_KEY_DOWN:
    case '>':
	return seek(60);
    case SL_KEY_PPAGE:
    case '9':
	return seek(-600);
    case SL_KEY_NPAGE:
    case '0':
	return seek(600);
    case SL_KEY_HOME:
    case '(':
	return (check_position != 0) && seek_absolute(0);
    case SL_KEY_END:
    case ')':
	if ((length > 0) && (check_position != length)) {
	    player->Pause();
	    return seek_absolute(length);
	} else
	    return 0;
    case '[':
    case ']':
    case '{':
    case '}':
	return clip_motion_command(key);
    case 'b':
	if (bandwidth > 0)
	    print_bandwidth();
	else
	    puts("Bandwidth is unknown");

	return 1;
    case 'c':
	clear_status_line();
	flags ^= COMPACT_OUTPUT;
	printf("Compact output %s\n",
	       (flags & COMPACT_OUTPUT) ? "on" : "off");
	return 1;
    case 'i':
	print_tac();
	return 1;
    case 'l':
	if (length >= 0)
	    print_length();
	else
	    puts("Length is unknown");

	return 1;
    case 'q':
	clear_status_line();
	flags ^= QUIET;
	printf("Quiet mode %s\n", (flags & QUIET) ? "on" : "off");
	return 1;
    case 's':
	if (flags & COMPACT_OUTPUT) {
	    clear_status_line();
	    print_status_line(1);
	} else
	    print_state();

	return 1;
    case 't':
	clear_status_line();

	if (flags & COMPACT_OUTPUT)
	    fputs("Time: ", stdout);
	else
	    fputs("Elapsed time is ", stdout);

	ULONG32 time;
	time = player->GetCurrentPlayTime();

	if (time > 0) {
	    print_time(time, flags & COMPACT_OUTPUT);

	    if (player->IsLive())
		fputs(" (live)", stdout);
	    else if (length > 0)
		printf(" (%.0f%%)", (time / (float) length) * 100);
	} else
	    fputs("none", stdout);

	putchar('\n');
	return 1;
    case 'u':
	int source_count;
	clear_status_line();
	source_count = player->GetSourceCount();

	if (source_count > 1)
	    puts("Multiple stream sources:");

	for (int i = 0; i < source_count; i++) {
	    IUnknown *source_unknown;
	    IRMAStreamSource *source;
	    player->GetSource(i, source_unknown);
	    source_unknown->QueryInterface(IID_IRMAStreamSource,
					   (void **) &source);
	    printf("%s\n", source->GetURL());
	    source->Release();
	    source_unknown->Release();
	}

	return 1;
    case 13:
    case 10:
	/* Allow a CR to pass through wtihout a beep when running in line
	   mode. */
	return ~flags & TTY_INITIALIZED;
    default:
	return 0;
    }
}

FPRMCLOSEENGINE close_engine;

  //char *url = NULL;
char *url = URL;		// XXX

void doit(IRMAClientEngine * engine, IRMAClientEngineSelector * selector,
	  time_t start_time)
{


    error = player->OpenURL(url);

    if (error == PNR_OK)
	error = player->Begin();

    if (error != PNR_OK) {
	error_handler->ErrorOccurred(0, error, 0, NULL, NULL);
	//return error;
	return;
    }

    engine->QueryInterface(IID_IRMAClientEngineSelector,
			   (void **) &selector);


    if (flags & INTERACTIVE)
	init_tty();

    signal(SIGINT, interrupted);
    signal(SIGTERM, interrupted);
    signal(SIGHUP, interrupted);
    start_time = time(NULL);
    time_t last_time = start_time;
    fflush(stdout);
    setvbuf(stdout, NULL, _IONBF, 0);

    while (~flags & STOPPED) {
	int key = 0;
	time_t current_time = time(NULL);

	if ((run_time >= 0) && (current_time >= (start_time + run_time)))
	    flags |= QUIT;

	if (flags & QUIT) {
	    quit_time = player->GetCurrentPlayTime();
	    player->Stop();
	}

	selector->Select(0, NULL, NULL, NULL, NULL);
	engine->EventOccurred(NULL);

	if (flags & TTY_INITIALIZED) {
	    if (SLang_input_pending(-10))
		key = SLkp_getkey();
	} else if (flags & INTERACTIVE) {
	    fd_set rfds;
	    struct timeval tv;

	    FD_ZERO(&rfds);
	    FD_SET(0, &rfds);
	    tv.tv_sec = 0;
	    tv.tv_usec = 10000;

	    if (select(1, &rfds, NULL, NULL, &tv))
		read(0, &key, 1);
	} else
	    usleep(10000);

	if (key) {
	    if (!run_command(key))
		putchar('\007');

	    if (flags & SHOW_STATUS_LINE)
		print_status_line(~flags & TTY_INITIALIZED);
	} else if ((flags & SHOW_STATUS_LINE)
		   && (last_time != current_time)) {
	    print_status_line(~flags & TTY_INITIALIZED);
	    last_time = current_time;
	}
    }

    reset_tty();

    if (flags & SHOW_STATUS_LINE)
	print_status_line(1);
    else if ((flags & QUIT) && (~flags & QUIET) && (quit_time >= 1000)) {
	fputs("Stopped at ", stdout);
	print_time(quit_time, flags & COMPACT_OUTPUT);
	putchar('\n');
    }

    close_engine(engine);
}

/***********************************************************************
 * 		Main
 ***********************************************************************/
int main(int argc, char **argv)
{
    XEvent report;
    XGCValues xgcValues;
    XTextProperty app_name_atom;

    FPRMCREATEENGINE create_engine;
    // FPRMCLOSEENGINE close_engine;
    FPRMSETDLLACCESSPATH set_dll_access_path;
    IUnknown *context = NULL;
    IRMAClientEngine *engine = NULL;
    IRMAClientEngineSelector *selector = NULL;
    char *rp_home = getenv("REALPLAYER_HOME");
    IRMAErrorSinkControl *error_sink_control = NULL;
    time_t start_time;
    char *path_next_position;

    int dummy = 0;
    int i;
    char Geometry_str[64] = "64x64+0+0";
    char Display_str[64] = "";

    int screen;
    XSizeHints xsizehints;
    XWMHints *xwmhints;
    XClassHint xclasshint;
    int N = 1;			/* Button number pressed to goto app #     */
    int mmouse = 1;
    int playing = 0;


    strcpy(configfile, getenv("HOME"));	// Added by Casey Harkin, 3/6/99
    strcat(configfile, CONFFILENAME);	// Fixed Bug - didn't look in home directory
    // but startup directory

    /* Parse Command Line Arguments */
    for (i = 1; i < argc; i++) {
	if (*argv[i] == '-') {
	    switch (*(argv[i] + 1)) {
	    case 'v':		// Turn on Verbose (debugging) Mode
		Verbose = 1;
		break;
	    case 'g':		// Set Geometry Options
		if (++i >= argc)
		    show_usage();
		sscanf(argv[i], "%s", Geometry_str);
		if (Verbose)
		    printf("Geometry is: %s\n", Geometry_str);
		break;
	    case 'd':		// Set display 
		if (++i >= argc)
		    show_usage();
		sscanf(argv[i], "%s", Display_str);
		if (Verbose)
		    printf("Display is: %s\n", Display_str);
		break;
	    case 'h':		// Show Help Message
		show_usage();
		break;
	    case 'f':		// use config file <filename>
		if (++i >= argc)
		    show_usage();
		sscanf(argv[i], "%s", configfile);
		if (Verbose)
		    printf("Using Config File: %s\n", configfile);
		break;
	    default:		// other, unknown, parameters
		show_usage();
		break;
	    }
	}
    }

    // REAL Start

    if (rp_home == NULL)
	rp_home = REALPLAYER_HOME;

    char *lib_path = (char *) malloc(strlen(rp_home) + 23);
    sprintf(lib_path, "%s/Common/rmacore.so.6.0", rp_home);
    void *lib = dlopen(lib_path, RTLD_LAZY);
    free(lib_path);


    if (lib == NULL) {
	fprintf(stderr, "Failed to load rmacore.so.6.0: %s\n", dlerror());
	return PNR_FAILED;
    }

    create_engine = (FPRMCREATEENGINE) dlsym(lib, "CreateEngine");
    close_engine = (FPRMCLOSEENGINE) dlsym(lib, "CloseEngine");
    set_dll_access_path =
	(FPRMSETDLLACCESSPATH) dlsym(lib, "SetDLLAccessPath");

    /* Tell the client core where the various shared libraries are stored.  The
       paths of the shared library directories are separated by null
       characters. */
    char paths[256];
    path_next_position = paths;
    memset(paths, 0, 256);
    char next_path[256];
    memset(next_path, 0, 256);
    sprintf(next_path, "DT_Common=%s/Common", rp_home);
    memcpy(path_next_position, next_path, strlen(next_path) + 1);
    path_next_position += strlen(next_path) + 1;
    sprintf(next_path, "DT_Plugins=%s/Plugins", rp_home);
    memcpy(path_next_position, next_path, strlen(next_path) + 1);
    path_next_position += strlen(next_path) + 1;
    sprintf(next_path, "DT_Codecs=%s/Codecs", rp_home);
    memcpy(path_next_position, next_path, strlen(next_path) + 1);
    set_dll_access_path(paths);

    if (create_engine(&engine) != PNR_OK)
	return PNR_FAILED;

    if (engine->CreatePlayer(player) != PNR_OK)
	return PNR_FAILED;

    player->QueryInterface(IID_IRMAPNRegistry, (void **) &registry);
    init_tac();
    context = new tr_context();
    player->SetClientContext(context);
    player->QueryInterface(IID_IRMAErrorSinkControl,
			   (void **) &error_sink_control);
    error_sink_control->AddErrorSink(error_handler, PNLOG_EMERG,
				     PNLOG_INFO);
    error_sink_control->Release();

    // REAL END

    if ((display = XOpenDisplay(Display_str)) == NULL) {
	fprintf(stderr, "Fail: XOpenDisplay for %s\n", Display_str);
	exit(-1);
    }

    screen = DefaultScreen(display);
    rootwin = RootWindow(display, screen);

    bg_pixel = WhitePixel(display, screen);
    fg_pixel = BlackPixel(display, screen);

    xsizehints.flags = USSize | USPosition;
    xsizehints.width = 64;
    xsizehints.height = 64;

    /* Parse Geometry string and fill in sizehints fields */
    XWMGeometry(display, screen,
		Geometry_str,
		NULL,
		border,
		&xsizehints,
		&xsizehints.x,
		&xsizehints.y,
		&xsizehints.width, &xsizehints.height, &dummy);

    if ((win = XCreateSimpleWindow(display,
				   rootwin,
				   xsizehints.x,
				   xsizehints.y,
				   xsizehints.width,
				   xsizehints.height,
				   border, fg_pixel, bg_pixel)) == 0) {
	fprintf(stderr, "Fail: XCreateSimpleWindow\n");
	exit(-1);
    }

    if ((iconwin = XCreateSimpleWindow(display,
				       win,
				       xsizehints.x,
				       xsizehints.y,
				       xsizehints.width,
				       xsizehints.height,
				       border, fg_pixel, bg_pixel)) == 0) {
	fprintf(stderr, "Fail: XCreateSimpleWindow\n");
	exit(-1);
    }

    /* Set up shaped windows */
    /*Gives the appicon a border so you can grab and move it. */

    if ((pixmask = XCreateBitmapFromData(display,
					 win,
					 mask_bits,
					 mask_width, mask_height)) == 0) {
	fprintf(stderr, "Fail: XCreateBitmapFromData\n");
    }

    XShapeCombineMask(display, win, ShapeBounding, 0, 0, pixmask,
		      ShapeSet);
    XShapeCombineMask(display, iconwin, ShapeBounding, 0, 0, pixmask,
		      ShapeSet);

    /* Convert in pixmaps from .xpm includes. */
    getPixmaps();

    /* Interclient Communication stuff */
    /* Appicons don't work with out this stuff */
    xwmhints = XAllocWMHints();
    xwmhints->flags = WindowGroupHint | IconWindowHint | StateHint;
    xwmhints->icon_window = iconwin;
    xwmhints->window_group = win;
    xwmhints->initial_state = WithdrawnState;
    XSetWMHints(display, win, xwmhints);

    xclasshint.res_name = "wmreal";
    xclasshint.res_class = "Wmreal";
    XSetClassHint(display, win, &xclasshint);

    XSetWMNormalHints(display, win, &xsizehints);

    /* Tell window manager what the title bar name is. We never see */
    /* this anyways in the WithdrawnState      */
    if (XStringListToTextProperty(&app_name, 1, &app_name_atom) == 0) {
	fprintf(stderr, "%s: Can't set up window name\n", app_name);
	exit(-1);
    }
    XSetWMName(display, win, &app_name_atom);

    /* Create Graphic Context */
    if ((gc =
	 XCreateGC(display, win, (GCForeground | GCBackground),
		   &xgcValues))
	== NULL) {
	fprintf(stderr, "Fail: XCreateGC\n");
	exit(-1);
    }

    /* XEvent Masks. We want both window to process X events */
    XSelectInput(display, win, ExposureMask | ButtonPressMask | ButtonReleaseMask |	/* added ButtonReleaseMask *charkins */
		 PointerMotionMask | StructureNotifyMask);
    XSelectInput(display, iconwin, ExposureMask | ButtonPressMask | ButtonReleaseMask |	/* added ButtonReleaseMask *charkins */
		 PointerMotionMask | StructureNotifyMask);

    /* Store the 'state' of the application for restarting */
    XSetCommand(display, win, argv, argc);

    /* Window won't ever show up until it is mapped.. then drawn after a  */
    /* ConfigureNotify */
    XMapWindow(display, win);

    /* X Event Loop */
    while (1) {

	if (playing) {
	    selector->Select(0, NULL, NULL, NULL, NULL);
	    engine->EventOccurred(NULL);
	}
	usleep(10000);

	if (XPending(display) || !playing) {
	    XNextEvent(display, &report);
	} else
	    continue;

	switch (report.type) {
	case Expose:
	    if (report.xexpose.count != 0) {
		break;
	    }
	    if (Verbose)
		fprintf(stdout, "Event: Expose\n");
	    redraw();
	    break;
	case ConfigureNotify:
	    if (Verbose)
		fprintf(stdout, "Event: ConfigureNotify\n");
	    redraw();
	    break;
	case ButtonPress:	/* draw button pressed, don't launch *charkins */
	    switch (report.xbutton.button) {
	    case Button1:
		N = whichButton(report.xbutton.x, report.xbutton.y);
		if ((N >= 0) && (N <= NUMAPPS)) {
		    button_pressed = N + LMASK;
		    redraw();
		}
		if (Verbose)
		    fprintf(stdout, "Button 1:x=%d y=%d N=%d\n",
			    report.xbutton.x, report.xbutton.y, N + LMASK);
		break;
	    case Button2:
		if (mmouse) {
		    N = whichButton(report.xbutton.x, report.xbutton.y);
		    if ((N >= 0) && (N <= NUMAPPS)) {
			button_pressed = N + MMASK;
			redraw();
		    }
		    if (Verbose)
			fprintf(stdout, "Button 1:x=%d y=%d N=%d\n",
				report.xbutton.x, report.xbutton.y,
				N + MMASK);
		}
		break;
	    case Button3:
		N = whichButton(report.xbutton.x, report.xbutton.y);
		if ((N >= 0) && (N <= NUMAPPS)) {
		    button_pressed = N + RMASK;
		    redraw();
		}
		if (Verbose)
		    fprintf(stdout, "Button 3:x=%d y=%d N=%d\n",
			    report.xbutton.x, report.xbutton.y, N + RMASK);
		break;
	    }
	    break;
	case ButtonRelease:	/* launch app here if still over button *charkins */
	    switch (report.xbutton.button) {
	    case Button1:
		N = whichButton(report.xbutton.x, report.xbutton.y);
		if ((N >= 0) && (N <= NUMAPPS) && (N == button_pressed)) {
		    if (N >= 1 && N <=8 && !playing) {
			fprintf(stderr,">>%s<<\n",Parse(N));
			error = player->OpenURL(Parse(N));

			if (error == PNR_OK)
			    error = player->Begin();

			engine->
			    QueryInterface(IID_IRMAClientEngineSelector,
					   (void **) &selector);

			playing = 1;

		    } else if (N == 9 && playing) {
			player->Stop();
			playing = 0;
		    } 
		}
		button_pressed = -1;
		redraw();
		if (Verbose)
		    fprintf(stdout, "Button 1:x=%d y=%d N=%d\n",
			    report.xbutton.x, report.xbutton.y, N + LMASK);
		break;
	    case Button2:
		if (mmouse) {
		    N = whichButton(report.xbutton.x, report.xbutton.y);
		if ((N >= 0) && (N <= NUMAPPS) && (N == button_pressed)) {
		    if (N >= 1 && N <=8 && !playing) {
			fprintf(stderr,">>%s<<\n",Parse(N + MMASK));
			error = player->OpenURL(Parse(N + MMASK));

			if (error == PNR_OK)
			    error = player->Begin();

			engine->
			    QueryInterface(IID_IRMAClientEngineSelector,
					   (void **) &selector);

			playing = 1;

		    } else if (N == 9 && playing) {
			player->Stop();
			playing = 0;
		    } 
		}
		    button_pressed = -1;
		    redraw();
		    if (Verbose)
			fprintf(stdout, "Button 1:x=%d y=%d N=%d\n",
				report.xbutton.x, report.xbutton.y,
				N + MMASK);
		}
		break;
	    case Button3:
		N = whichButton(report.xbutton.x, report.xbutton.y);
		if ((N >= 0) && (N <= NUMAPPS) && (N == button_pressed)) {
		    if (N >= 1 && N <=8 && !playing) {
			fprintf(stderr,">>%s<<\n",Parse(N + RMASK));
			error = player->OpenURL(Parse(N + RMASK));

			if (error == PNR_OK)
			    error = player->Begin();

			engine->
			    QueryInterface(IID_IRMAClientEngineSelector,
					   (void **) &selector);

			playing = 1;

		    } else if (N == 9 && playing) {
			player->Stop();
			playing = 0;
		    } 
		}
		button_pressed = -1;
		redraw();
		if (Verbose)
		    fprintf(stdout, "Button 3:x=%d y=%d N=%d\n",
			    report.xbutton.x, report.xbutton.y, N + RMASK);
		break;
	    }
	    break;
	case DestroyNotify:
	    if (Verbose)
		fprintf(stdout, "Bye\n");
	    XFreeGC(display, gc);
	    XDestroyWindow(display, win);
	    XDestroyWindow(display, iconwin);
	    XCloseDisplay(display);
	    if (Verbose)
		fprintf(stdout, "Bye\n");
	    exit(0);
	    break;
	}
    }
    return (0);
}

/***********************************************************************
 *   redraw
 *
 *	 Map the button region coordinates.
 *
 *   Draw the appropriate number of buttons on the 'visible' Pixmap 
 *   using data from the 'buttons' pixmap.
 *
 *   Then, copy the 'visible' pixmap to the two windows ( the withdrawn
 *   main window and the icon window which is the main window's icon image.)
 ***********************************************************************/
void redraw()
{
    int n;
    int i, j;
    int dest_x, dest_y;
    int space;
    int offset;
    int bsize = 18;

    space = 0;
    offset = 5;
    XCopyArea(display, templt.pixmap, visible.pixmap, gc, 0, 0,
	      templt.attributes.width, templt.attributes.height, 0, 0);

    for (j = 0; j < 3; j++) {
	for (i = 0; i < 3; i++) {
	    n = i + j * 3;
	    dest_x = i * (bsize + space) + offset + space;
	    dest_y = j * (bsize + space) + offset + space;

	    /* Define button mouse coords */
	    button_region[n].x = dest_x;
	    button_region[n].y = dest_y;
	    button_region[n].i = dest_x + bsize - 1;
	    button_region[n].j = dest_y + bsize - 1;

	    /* Copy button images for valid apps */
	    if ((n + 1) <= NUMAPPS) {
		XCopyArea(display, buttons.pixmap, visible.pixmap, gc,
			  i * bsize, j * bsize, bsize, bsize, dest_x,
			  dest_y);
	    }
	}
    }

    if (button_pressed > 0) {	/* draw pressed button *charkins */
	if (button_pressed > RMASK)
	    button_pressed -= RMASK;
	else if (button_pressed > MMASK)
	    button_pressed -= MMASK;
	else if (button_pressed > LMASK)
	    button_pressed -= LMASK;
	i = (button_pressed - 1) % 3;	/* get col of button */
	j = (button_pressed - 1) / 3;	/* get row of button */
	dest_x = i * (bsize + space) + offset + space;
	dest_y = j * (bsize + space) + offset + space;
	XSetForeground(display, gc, bg_pixel);
	XDrawLine(display, visible.pixmap, gc,
		  dest_x + 1, dest_y + bsize - 1, dest_x + bsize - 1,
		  dest_y + bsize - 1);
	XDrawLine(display, visible.pixmap, gc, dest_x + bsize - 1,
		  dest_y + bsize - 1, dest_x + bsize - 1, dest_y + 1);
	XSetForeground(display, gc, fg_pixel);
	XDrawLine(display, visible.pixmap, gc,
		  dest_x, dest_y, dest_x + bsize - 2, dest_y);
	XDrawLine(display, visible.pixmap, gc,
		  dest_x, dest_y, dest_x, dest_y + bsize - 2);
    }
    /*charkins */
    flush_expose(win);
    XCopyArea(display, visible.pixmap, win, gc, 0, 0,
	      visible.attributes.width, visible.attributes.height, 0, 0);
    flush_expose(iconwin);
    XCopyArea(display, visible.pixmap, iconwin, gc, 0, 0,
	      visible.attributes.width, visible.attributes.height, 0, 0);
    if (Verbose)
	fprintf(stdout, "In Redraw()\n");
}

/***********************************************************************
 *  whichButton
 *
 *  Return the button that at the x,y coordinates. The button need not
 *  be visible ( drawn ). Return -1 if no button match.
 ***********************************************************************/
int whichButton(int x, int y)
{
    int index;

    for (index = 0; index < NUMAPPS; index++) {
	if (x >= button_region[index].x &&
	    x <= button_region[index].i &&
	    y >= button_region[index].y && y <= button_region[index].j) {
	    return (index + 1);
	}
    }
    return (-1);
}

/***********************************************************************
 * RunAppN(int app)
 *
 * Run the command given in the configuration file 'configfile'
 ***********************************************************************/
void RunAppN(int app)
{
    char *cmndstr;

    cmndstr = Parse(app);	// Get command to pass to system
    if (Verbose)
	fprintf(stderr, "Command String: %s", cmndstr);
    if (cmndstr != NULL)
	system(cmndstr);	// if there's a command, run it
}

/***********************************************************************
 * Parse(int app)
 *
 * Parses the file 'configfile' for command to execute.
 ***********************************************************************/

char *Parse(int app)
{
    FILE *fp;
    char *ptr = NULL;
    char *cmnd = NULL;
    char *line = NULL;

    if ((fp = fopen(configfile, "r")) == NULL) {	// Can't find user config file
	strcpy(configfile, CONFIGGLOBAL);
	if ((fp = fopen(configfile, "r")) == NULL) {	// Can't find global config file
	    fprintf(stderr, "%s: Configuration File not found\n",
		    configfile);
	    exit(-1);
	}
    }

    do {			// Read Lines in config file
	line = readln(fp);
	if (line == NULL)
	    break;		// if end of file, quit
	if ((line[0] != '#') && (line[0] != '\n'))	// Ignore comments and blanks 
	    if (atoi(line) == app)
		break;		// if we've found our app, quit looking
    } while (1);

    if (line == NULL)
	return (NULL);		// if we didn't find in, that just ignore

    ptr = strchr(line, '\t');	// find first tab
    if (ptr == NULL)
	ptr = strchr(line, ' ');	// or space charater
    if (ptr == NULL)
	return (NULL);		// if neither are there, its a badly
    // formatted line.  Ignore
    ptr++;
    cmnd = (char *) malloc((strlen(ptr) + 1) * sizeof(char));

    strcpy(cmnd, ptr);		// Everything after number is passed to system

    free(line);
    fclose(fp);
    cmnd[strlen(cmnd)-1] = '\0';
    return (cmnd);
}

/***********************************************************************
 *   getPixmaps
 *    
 *   Load XPM data into X Pixmaps.
 *  
 *   Pixmap templt contains the untouched window backdrop image.
 *   Pixmap visible is the templt pixmap with buttons drawn on it.
 *          -- what is seen by the user.
 *   Pixmap buttons holds the images for individual buttons that are
 *          later copied onto Pixmap visible.
 ***********************************************************************/
void getPixmaps()
{
    templt.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
    visible.attributes.valuemask |=
	(XpmReturnPixels | XpmReturnExtensions);
    buttons.attributes.valuemask |=
	(XpmReturnPixels | XpmReturnExtensions);

    /* Template Pixmap. Never Drawn To. */
    if (XpmCreatePixmapFromData(display, rootwin, backdrop_xpm,
				&templt.pixmap, &templt.mask,
				&templt.attributes) != XpmSuccess) {
	fprintf(stderr, "Can't Create 'templt' Pixmap\n");
	exit(1);
    }
    /* Visible Pixmap. Copied from templt Pixmap and then drawn to. */
    if (XpmCreatePixmapFromData(display, rootwin, backdrop_xpm,
				&visible.pixmap, &visible.mask,
				&visible.attributes) != XpmSuccess) {
	fprintf(stderr, "Can't Create 'visible' Pixmap");
	exit(1);
    }

    /* Button Pixmap.  */
    if (XpmCreatePixmapFromData(display, rootwin, wmreal_xpm,
				&buttons.pixmap, &buttons.mask,
				&buttons.attributes) != XpmSuccess) {
	fprintf(stderr, "Can't Create 'buttons' Pixmap");
	exit(1);
    }
}

/***********************************************************************
 *   flush_expose
 *
 *   Everyone else has one of these... Can't hurt to throw it in.
 ***********************************************************************/
int flush_expose(Window w)
{
    XEvent dummy;
    int i = 0;

    while (XCheckTypedWindowEvent(display, w, Expose, &dummy))
	i++;
    return (i);
}

/***********************************************************************
 * 		show_usage
 ***********************************************************************/
void show_usage()
{
    fprintf(stderr, "\n");
    fprintf(stderr,
	    "usage: %s [-g geometry] [-d dpy] [-v] [-f <configfile>]\n",
	    app_name);
    fprintf(stderr, "\n");
    fprintf(stderr, " wmreal Ver %d Rel %d\n", VER, REL);
    fprintf(stderr, "\n");
    fprintf(stderr,
	    "-g  <geometry>   Window Geometry - ie: 64x64+10+10\n");
    fprintf(stderr, "-d  <display>    Display -  ie: 127.0.0.1:0.0\n");
    fprintf(stderr, "-f  <filename>   Full path to configuration file.\n");
    fprintf(stderr, "-v               Verbose Mode. \n");
    fprintf(stderr, "-h               Help. This message.\n");
    fprintf(stderr,
	    "-m               Enable Middle Mouse functionality.\n\n");
    exit(-1);
}
